<?php

namespace FiloBlu\Refilo\Model;

use FiloBlu\Refilo\Api\MergeCartsInterface;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\QuoteGraphQl\Model\Cart\GetCartForUser;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;

/**
 * Class MergeCart
 * @package FiloBlu\Refilo\Model
 */
class MergeCart implements MergeCartsInterface
{
    /**
     * @var GetCartForUser
     */
    private $getCartForUser;

    /**
     * @var CartRepositoryInterface
     */
    private $cartRepository;
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;
    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @param GetCartForUser $getCartForUser
     * @param StoreManagerInterface $storeManager
     * @param CartRepositoryInterface $cartRepository
     * @param LoggerInterface $logger
     */
    public function __construct(
        GetCartForUser $getCartForUser,
        StoreManagerInterface $storeManager,
        CartRepositoryInterface $cartRepository,
        LoggerInterface $logger
    ) {
        $this->getCartForUser = $getCartForUser;
        $this->cartRepository = $cartRepository;
        $this->storeManager = $storeManager;

        $this->logger = $logger;
    }

    /**
     * @param int $userId
     * @param string $storeCode
     * @param string $customerCartId
     * @param string $guestCartId
     * @return void
     */
    public function mergeGuestCartIntoActiveCustomerCart($userId, $storeCode, $customerCartId, $guestCartId)
    {
        try {
            $storeId = $this->getStoreId($storeCode);

            // passing customerId as null enforces source cart should always be a guestcart
            $guestCart = $this->getCartForUser->execute($guestCartId, null, $storeId);
            $customerCart = $this->getCartForUser->execute($customerCartId, $userId, $storeId);
            $customerCart->merge($guestCart);
            $guestCart->setIsActive(false);
            $this->cartRepository->save($customerCart);
            $this->cartRepository->save($guestCart);
        } catch (\Throwable $throwable) {
            $this->logger->error($throwable->getMessage(), ['exception' => $throwable]);
        }
    }

    /**
     * @param $storeCode
     * @return int
     */
    public function getStoreId($storeCode) : int
    {
        $stores = $this->storeManager->getStores(true, true);

        foreach ($stores as $code => $store) {
            if ($code === $storeCode) {
                return $store->getId();
            }
        }

        return 0;
    }
}
