<?php

namespace FiloBlu\Refilo\Remote\Connector;

use Elasticsearch\Client;
use Elasticsearch\ClientBuilder;
use phpDocumentor\Reflection\Types\String_;
use Traversable;

/**
 * Class ElasticSearchConnector
 * @package FiloBlu\Refilo\Remote\Connector
 */
class BumbleSearch extends AbstractConnector
{
    /**
     * @var Client
     */
    protected $handler;

    /**
     * @var string
     */
    protected $token;

    /**
     * @var string
     */
    protected $refresh_token;

    /**
     * @var string
     */
    protected $auth_url = "https://welcome-ph3ofm4rwa-ew.a.run.app";

    /**
     * @return mixed|void
     */
    public function disconnect(): ConnectorInterface
    {
        if (!$this->handler) {
            return $this;
        }

        $this->handler = null;

        return $this;
    }

    /**
     * @param Traversable $entities
     * @param string $collection
     * @return mixed|void
     */
    public function create(Traversable $entities, string $collection)
    {
        $this->createIndex($collection);

        $client = $this->getHandler();

        $i = 1;

        try {

            foreach ($entities as $entity) {

                $doc = $entity->toArray();
                $doc['id'] = (string)$entity->getId();

                $docs[] = $doc;

                if ($i % 500 === 0) {
                    $response = $client->request('POST', 'indexes/'.$collection.'/bulk', [
                        'json'    => $docs,
                        'headers' => $this->getHeaders()
                    ]);
                    $docs = [];
                }
                $i++;


            }

            if (!empty($docs)) {
                $response = $client->request('POST', 'indexes/'.$collection.'/bulk', [
                    'json'    => $docs,
                    'headers' => $this->getHeaders()
                ]);
            }

            $this->mapFields($collection);


        } catch (\Exception $e) {

            echo $e->getMessage();

        }


    }


    private function buildFilterMapping($fields) {

        $matches  = preg_grep ('/^filters.(.*).value/i', $fields);

        $mapping = [];

        if (count($matches)) {
            foreach ($matches as $item) {

                $attribute = explode('.',$item);

                if ($attribute[1] === 'price')
                    continue;

                $field_template = [
                    "name" => $item,
                    "label" => ucwords($attribute[1]),
                    "code" => $attribute[1],
                    "facet" => true,
                    "facet_size" => 20,
                    "filterable" => true,
                    "sortable" => false,
                    "searchable" => false,
                    "filterable_type" => "values",
                    "filterable_range_step" => 0,
                    "filterable_priority" => 0,
                    "sortable_priority" => 0,
                    "searchable_priority" => 0
//                    "sub_filters" => [
//                        [
//                            "label" => "Title",
//                            "name" => "filters." . $attribute[1] . ".title"
//                        ],
//                        [
//                            "label" => "Title",
//                            "name" => "filters." . $attribute[1] . ".image"
//                        ]
//                    ]
                ];

                $mapping[$attribute[1]] = $field_template;

            }

        }

        return $mapping;

    }

    private function buildSearchableMapping($fields,$mapping) {

        $matches  = ["attributes.name"];

        if (count($matches)) {
            foreach ($matches as $item) {

                $attribute = explode('.',$item);

                if (isset($mapping[$attribute[1]])) {

                    $mapping[$attribute[1]]["searchable"] = true;
                }

                else {

                    $field_template = [
                        "name" => $item,
                        "label" => ucwords($attribute[1]),
                        "code"  => $attribute[1],
                        "facet" => false,
                        "facet_size" => 20,
                        "filterable" => false,
                        "sortable" => false,
                        "searchable" => true,
                        "filterable_type" => "values",
                        "filterable_range_step" => 0,
                        "filterable_priority" => 0,
                        "sortable_priority" => 0,
                        "searchable_priority" => 0
                    ];

                    $mapping[$attribute[1]] = $field_template;

                }



            }

        }

        return $mapping;

    }

    private function buildSortableMapping($fields,$mapping) {

        $matches  = preg_grep ('/^sortables.(.*).value/i', $fields);

        if (count($matches)) {
            foreach ($matches as $item) {

                $attribute = explode('.',$item);

                if (isset($mapping[$attribute[1]])) {

                    $mapping[$attribute[1]]["sortable"] = true;
                }

                else {

                    $field_template = [
                        "name" => $item,
                        "label" => ucwords($attribute[1]),
                        "code"  => $attribute[1],
                        "facet" => false,
                        "facet_size" => 20,
                        "filterable" => false,
                        "sortable" => true,
                        "searchable" => false,
                        "filterable_type" => "values",
                        "filterable_range_step" => 0,
                        "filterable_priority" => 0,
                        "sortable_priority" => 0,
                        "searchable_priority" => 0
                    ];

                    $mapping[$attribute[1]] = $field_template;

                }



            }

        }

        return $mapping;

    }

    private function buildPriceMapping($fields,$mapping) {

        $matches  = preg_grep ('/^pricing_.(.*).final_price/i', $fields);

        if (count($matches)) {
            foreach ($matches as $item) {

                $attribute = explode('.',$item);

                $field_template = [
                    "name" => $item,
                    "label" => 'Price',
                    "code"  => $attribute[0],
                    "facet" => true,
                    "facet_size" => 20,
                    "filterable" => true,
                    "sortable" => false,
                    "searchable" => false,
                    "filterable_type" => "range",
                    "filterable_range_step" => 10,
                    "filterable_priority" => 0,
                    "sortable_priority" => 0,
                    "searchable_priority" => 0
                ];

                $mapping[$attribute[0]] = $field_template;

            }

        }

        return $mapping;

    }


    private function mapFields($index) {

        try {
            $client = $this->getHandler();
            $response = $client->request('GET', 'indexes/'.$index . '/fields', [
                'headers' => $this->getHeaders()
            ]);

            $fields = $this->decodeResponse($response);

            $mapping = $this->buildFilterMapping($fields);
            $mapping = $this->buildPriceMapping($fields,$mapping);
            $mapping = $this->buildSortableMapping($fields,$mapping);
            $mapping = $this->buildSearchableMapping($fields,$mapping);


            foreach ($mapping as $map) {

                $response = $client->request('POST', 'indexes/'.$index. '/mapping/field', [
                    'headers' => $this->getHeaders(),
                    'json' => $map
                ]);

            }

        } catch (\Exception $e) {
            echo $e->getMessage();
        }



    }

    /**
     * @param $index
     * @return $this
     */
    public function createIndex($index)
    {

        $client = $this->getHandler();
        try {
            $response = $client->request('GET', 'indexes', [
                'headers' => $this->getHeaders()
            ]);

            $indices = $this->decodeResponse($response);

            if (array_key_exists($index, $indices)) {
                return $this;
            }

            $response = $client->request('POST', 'indexes/'.$index, [
                'headers' => $this->getHeaders()
            ]);

        } catch (\Exception $e) {
            throw new \Error($e);
        }

        return $this;
    }

    private function getHeaders() {
        return [
            'Authorization' => 'Bearer ' . $this->token,
            'Accept'        => 'application/json',
        ];
    }

    /**
     * @param $response
     * @return \GuzzleHttp\Psr7\Stream
     */
    private function decodeResponse($response) {
        return json_decode($response->getBody());
    }


    protected function getHandler()
    {

        $this->connect();

        return $this->handler;
    }

    /**
     * @return ConnectorInterface
     */
    public function connect(): ConnectorInterface
    {

        if (!$this->handler) {
            $this->handler = new \GuzzleHttp\Client();
            $hosts = $this->getHosts();

            $connection = $this->decodeHost($hosts[0]);

            $r = $this->handler->request('POST',
                $this->auth_url.'/account/token',
                $connection["auth"]);

            $response = json_decode((string)$r->getBody());

            $this->token = $response->accessToken;
            $this->refresh_token = $response->refreshToken;

            $this->handler = new \GuzzleHttp\Client(['base_uri' => $connection["host"]]);

        }
        return $this;
    }

    private function decodeHost($connection_string) {

        $connection_parts = explode('|',$connection_string);

        $auth_parts = (isset($connection_parts[0])) ? explode(":",$connection_parts[0]) : [];

        $connection["host"] = (isset($connection_parts[1])) ? $connection_parts[1] : '';
        $connection["auth"] = ['json' => [
            "appId" => $auth_parts[0],
            "accountId" => isset($auth_parts[1]) ? $auth_parts[1] : '',
            "secret" => isset($auth_parts[2]) ? $auth_parts[2] : '',
        ]];

        return $connection;

    }

    /**
     * @return array
     */
    protected function getHosts(): array
    {
        $hosts = $this->getConfiguration()->getData(self::CONFIGURATION_HOST_POOL);

        $dsns = [];
        $dns = '';
        foreach ($hosts as $host) {

            if ($host['username']) {
                $dsn = "{$host['username']}:{$host['password']}|";
            }

            $dsn .= "{$host['scheme']}://";

            $dsn .= $host['host'];

            if ($host['port']) {
                $dsn .= ":{$host['port']}";
            }
            $dsns[] = $dsn;
        }

        return $dsns;
    }

    /**
     * @param array $entity
     * @param string $collection
     * @return mixed|void
     */
    public function delete(array $entity, string $collection)
    {
    }

    /**
     * @param Traversable $entities
     * @param string $collection
     * @return mixed|void
     */
    public function update(\Traversable $entities, string $collection)
    {

        $this->create($entities,$collection);



    }

    /**
     * @return array|mixed|null
     */
    protected function getPrefix()
    {
        return $this->getConfiguration()->getData('prefix');
    }
}
