<?php

namespace FiloBlu\Refilo\Remote\Entity\Provider;

use FiloBlu\Refilo\Remote\Entity\CategoriesProduct;
use FiloBlu\Refilo\Remote\Entity\EntityProviderInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DB\Select;
use Magento\Framework\ObjectManagerInterface;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Api\Data\WebsiteInterface;
use Magento\Store\Model\StoreManagerInterface;
use Zend_Db;

/**
 * Class StockProvider
 * @package FiloBlu\Refilo\Remote\Entity\Provider
 */
class CategoriesProductProvider implements EntityProviderInterface
{

    /** @var null */
    protected $ids;
    /** @var int */
    protected $website;
    /** @var array */
    protected $items = [];

    /** @var ResourceConnection */
    private $resourceConnection;

    /**
     * @var ObjectManagerInterface
     */
    private $objectManager;
    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * CategoriesProductProvider constructor.
     * @param ObjectManagerInterface $objectManager
     * @param ResourceConnection $resourceConnection
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        ObjectManagerInterface $objectManager,
        ResourceConnection $resourceConnection,
        StoreManagerInterface $storeManager
    ) {
        $this->resourceConnection = $resourceConnection;
        $this->objectManager = $objectManager;
        $this->storeManager = $storeManager;
    }

    /**
     * @return mixed
     */
    public function current()
    {
        $current = current($this->items);

        $productCategories = new CategoriesProduct();
        $productCategories->setId((int)key($this->items));

        foreach ($current as $name => $position) {
            $productCategories->setData($name, $position);
        }


        return $productCategories;
    }

    /**
     * @return mixed|void
     */
    public function next()
    {
        return next($this->items);
    }

    public function key()
    {
        return current($this->items)['id'];
    }

    public function valid()
    {
        return current($this->items);
    }

    /**
     *
     */
    public function rewind()
    {
        $select = $this->getSelectForProducts();

        if (!empty($this->ids)) {
            $select->where('s.category_id IN (' . implode(',', $this->ids) . ')');
        }

        $items = $this->resourceConnection->getConnection()->fetchAll($select, [], Zend_Db::FETCH_ASSOC);

        foreach ($items as $item) {
            $this->items[$item["product_id"]]["category_" . $item["category_id"]] = (int)$item["position"];
        }
    }

    /**
     * @return Select
     */
    protected function getSelectForProducts()
    {
        return $this->resourceConnection->getConnection()
            ->select()
            ->from(
                [
                    's' => "catalog_category_product"
                ],
                [
                    'id'          => 's.entity_id',
                    'category_id' => 's.category_id',
                    'product_id'  => 's.product_id',
                    'position'    => 's.position'
                ]
            );
    }

    /**
     * @param array $ids
     * @return $this|EntityProviderInterface
     */
    public function withIds(array $ids): EntityProviderInterface
    {
        $this->ids = $ids;
        return $this;
    }

    /**
     * @param StoreInterface $store
     * @return $this|EntityProviderInterface
     */
    public function forStore(StoreInterface $store): EntityProviderInterface
    {
        return $this;
    }

    /**
     * @param WebsiteInterface $website
     * @return $this|EntityProviderInterface
     */
    public function forWebsite(WebsiteInterface $website): EntityProviderInterface
    {
        return $this;
    }
}
